/*
 *  PseudoPS -- a small PostScript interpreter (1987)
 *
 *  Written by		Craig E Rasmussen
 *					Center for Atmospheric and Space Science
 *					Utah State University
 *					Logan, Utah 84322-4405
 *					(801) 750-2967
 *
 *	email		-	cer@star.stanford.edu
 *				-	theory::craig on the SPAN network
 */

#include <Quickdraw.h>
#include <stdio.h>
#include "PseudoPS.h"

#define DTR .01745329252

float xShift, yShift, xScale, yScale, rotate, cosTH, sinTH;
int HandleAllocated = FALSE, RegionOpen = FALSE;
RgnHandle region;


ParsePS(s)
char *s;
{
	Rect rect;
	float x1, x2, x3, x4, x5, left, top, right, bottom;
	int pop();
	char outstring[80];

	switch (*s) {
		case 'a':
			if (strcmp(s,"arc") == 0) {
				if (pop(&x5) != 0) StackError();
				if (pop(&x4) != 0) StackError();
				if (pop(&x3) != 0) StackError();
				if (pop(&x2) != 0) StackError();
				if (pop(&x1) != 0) StackError();
				left   = x1 - x3;
				top    = x2 + x3;
				right  = x1 + x3;
				bottom = x2 - x3;
				transform(&left, &top);
				transform(&right, &bottom);
				SetRect(&rect,(int)left,(int)top,(int)right,(int)bottom);
				FrameArc(&rect, (int)(90.-x4), (int)(x4-x5));
			}
			else PSerror(s);
			break;
		case 'c':
			if (strcmp(s,"closepath") == 0) {
				if (!HandleAllocated) PSerror("closepath (no RegionHandle)");
				else if (!RegionOpen) PSerror("closepath (no current path)");
				else {
					CloseRgn(region);
					RegionOpen = FALSE;
				}
			}
			else PSerror(s);
			break;
		case 'f':
			if (strcmp(s,"fill") == 0) {
				if (!HandleAllocated) PSerror("fill (no RegionHandle)");
				else {
					PaintRgn(region);
					DisposeRgn(region);
					HandleAllocated = FALSE;
				}
			}
			else PSerror(s);
			break;
		case 'l':
			if (strcmp(s,"lineto") == 0) {
				if (pop(&x2) != 0) StackError();
				if (pop(&x1) != 0) StackError();
				transform(&x1, &x2);
				LineTo((int)x1, (int)x2);
			}
			else PSerror(s);
			break;
		case 'm':
			if (strcmp(s,"moveto") == 0) {
				if (pop(&x2) != 0) StackError();
				if (pop(&x1) != 0) StackError();
				transform(&x1, &x2);
				MoveTo((int)x1, (int)x2);
			}
			else PSerror(s);
			break;
		case 'n':
			if (strcmp(s,"newpath") == 0) {
				if (HandleAllocated) {
					if (RegionOpen) {
						PSerror("newpath (region already open)");
						CloseRgn(region);
					}
					PSerror("newpath (RegionHandle already created)");
					DisposeRgn(region);
				}
				region = NewRgn();
				OpenRgn();
				HandleAllocated = TRUE;
				RegionOpen = TRUE;
			}
			else PSerror(s);
			break;
		case 'r':
			if (strcmp(s,"rlineto") == 0) {
				if (pop(&x2) != 0) StackError();
				if (pop(&x1) != 0) StackError();
				scale(&x1, &x2);
				Line((int)x1, (int)x2);
			}
			else if (strcmp(s,"rmoveto") == 0) {
				if (pop(&x2) != 0) StackError();
				if (pop(&x1) != 0) StackError();
				scale(&x1, &x2);
				Move((int)x1, (int)x2);
			}
			else if (strcmp(s,"rotate") == 0) {
				if (pop(&x1) != 0) StackError();
				rotate = x1;
				cosTH = 0.0;  /*cos(rotate*DTR);*/
				sinTH = 1.0;  /*sin(rotate*DTR);*/
			}
			else PSerror(s);
			break;
		case 's':
			if (strcmp(s,"scale") == 0) {
				if (pop(&x2) != 0) StackError();
				if (pop(&x1) != 0) StackError();
				xScale *= x1;
				yScale *= x2;
			}
			else if (strcmp(s,"scalefont") == 0) {
				if (pop(&x1) != 0) StackError();
				TextSize((int)x1);
			}
			else if (strcmp(s,"setgray") == 0) {
				if (pop(&x1) != 0) StackError();
				if (x1 > .875) PenPat(white);
				else if (x1 > .625) PenPat(ltGray);
				else if (x1 > .375) PenPat(gray);
				else if (x1 > .125) PenPat(dkGray);
				else PenPat(black);
			}
			else if (strcmp(s,"setlinewidth") == 0) {
				if (pop(&x1) != 0) StackError();
				PenSize((int)x1, (int)x1);
			}
			else if (strcmp(s,"show") == 0) {
				CtoPstr(PStext);
				DrawString(PStext);
			}
			else if (strcmp(s,"showpage") == 0) break;		/* nop */
			else if (strcmp(s,"stroke") == 0) {
				if (!HandleAllocated) PSerror("stroke (no RegionHandle)");
				else {
					FrameRgn(region);
					DisposeRgn(region);
					HandleAllocated = FALSE;
				}
			}
			else PSerror(s);
			break;
		case 't':
			if (strcmp(s,"translate") == 0) {
				if (pop(&x2) != 0) StackError();
				if (pop(&x1) != 0) StackError();
				xShift += x1;
				yShift += x2;
			}
			else PSerror(s);
			break;
		default:
			PSerror(s);
	}
}

PSerror(s)
char *s;
{
/*	SerialPutS("\n%offending command -> ");
	SerialPutS(s);
	SerialPutChar('\n');  */
	fprintf(fpErr, "%%offending command -> %s\n", s);
}


StackError()
{
/*	SerialPutS("\n%stack error\n");  */
	fprintf(fpErr, "\n%%stack error\n");
}


transform(x, y)
float *x, *y;
{
	*x += xShift;
	*y += yShift;	
	scale(x, y);
}


scale1D(r)
float *r;
{
	*r *= sqrt(xScale*xScale + yScale*yScale);
}


TransformAngle(t)
float *t;
{
	*t += rotate;
}


scale(x, y)
float *x, *y;
{
	*x *=  xScale;
	*y *= -yScale;					/* change increasing y to upwards */
}

SetTransforms(wXtop, wYtop, wXbot, wYbot)
int wXtop, wYtop, wXbot, wYbot;
{
	xShift = 0.0; yShift = 0.0; xScale = 1.0; yScale = 1.0;
	rotate = 0.0; cosTH = 1.0; sinTH = 0.0;
	SetOrigin(0, wYtop - wYbot);
	PenPat(black);
	PenSize(1,1);
}

