/*
 *  PseudoPS -- a small PostScript interpreter (1987)
 *
 *  Written by		Craig E Rasmussen
 *					Center for Atmospheric and Space Science
 *					Utah State University
 *					Logan, Utah 84322-4405
 *					(801) 750-2967
 *
 *	email		-	cer@star.stanford.edu
 *				-	theory::craig on the SPAN network
 *
 *  with the use of SimpleTools.c (c) Erik Kilk 1986
 * 
 *  This program may be freely distributed and modified as long as this
 *  header remains in place.
 *
 *  Link with LightSpeedC modules
 *	  -	MacTraps
 *	  -	math
 *	  -	stdio
 *	  -	strings
 *	  -	unix
 */
 
#include <Quickdraw.h>
#include "simple.h"		/* SimpleTools header file	*/
#include "PseudoPS.h"

FILE *fpPS, *fpErr;
char PStext[STRINGLENGTH];
int PSopen = FALSE;

inwindow (x, y)			/* executed when click in our window */
int x, y;
{
  Point m;
	while ( StillDown() ) {	/* while the Button is pressed */
		GetMouse (&m);		/* waste time */
	}

}

nop()
{
}

no_edit ()			/* turn off edit menu (on activation) */
{
  menu ("Edit", "", itemdisable);
}

yes_edit ()			/* turn on edit menu (on deactivation) */
{
  menu ("Edit", "", itemenable);
}

aboutPs()			/* About message */
{
	char messageStr[255];
	strcpy(messageStr,"PseudoPS -- by Craig E Rasmussen (1987)\r");
	strcat(messageStr,"A very small PostScript interpreter.\r");
	strcat(messageStr,"Please freely distribute and improve.\r");
	if (message(messageStr)) {
		strcpy(messageStr,"Programmed with the aid of SimpleTools\r");
		strcat(messageStr,"(c) Erik Kilk 1986");
		message(messageStr);
	}
}

OpenPSfile()			/* to be executed File-open menu is selected */
{
	char PSfile[255], *PtoCstr();
	int i, n, c;
	
	if (PSopen == TRUE) {
		SysBeep(10);
		message("a file is already open");
		return;
	}
	if (getfile("TEXT", PSfile)) {
		if ((fpPS = fopen(PtoCstr(PSfile), "r")) != NULL) {
			PSopen = TRUE;
			window (PSfile, WXTOP, WYTOP, WXBOT, WYBOT
				, no_edit, yes_edit, nop, inwindow);
			menu ("File", "Open.../O", itemdisable);
			SetTransforms(WXTOP, WYTOP, WXBOT, WYBOT);
		}
		else SysBeep(10);
	}
	else SysBeep(10);
}


setup ()			/* Setup the menus and windows */
{
	menu (applestring, "About PsuedoPS...", aboutPs);
	menu (applestring, "About PsuedoPS...", itemenable);
	menu ("File", "Open.../O", OpenPSfile);
	simplequits ();
	
	if ((fpErr = fopen("PS.errors", "w")) == NULL) {
		SysBeep(0);
		exit();
	}
}

main ()
{
	
	simpletools ("About PsuedoPS...");		/* Initialize SimpleTools	*/
	setup ();
	
	while (1) {
		simpleevents ();					/* Handle all events		*/
		PostScript();
	}
}


PostScript()
{
	char s[STRINGLENGTH];
	int status;
	float atof();

	if (!PSopen) return;	
	if ((status = getPS(s, STRINGLENGTH))) {
		if (status == -1)
			fprintf(fpErr, "PostScript buffer overflow\n");
		else if (status == -2) {
			fprintf(fpErr, "unterminated PostScript string\n");
			strcpy(PStext, s);
		}
		else if (status == 2) strcpy(PStext, s);
		else if (IsInteger(s)) push( (float)atoi(s) );
		else if (IsFloat(s)) push(atof(s));
		else ParsePS(s);
	}
}


/*
 *  Copy PostScript token to s if present, return(1); else return(0) if
 *  no token present or return(-1) if buffer overflow.
 */
 
getPS(s, max)
char *s;
int max;
{
	char *sPtr;
	int count = 0, c;

	sPtr = s;
	while (WhiteSpace(c = getc(fpPS)));		/* remove leading blanks */

	if (c == '(') return(getPSstr(s, max));	/* get PostScript string */
	
	while (!WhiteSpace(c)) {				/* copy character to s */
		if (c != EOF) {
			if (++count < max) *sPtr++ = c;	/* copy c if space available */
			else {							/* buffer overflow */
				*sPtr = '\0';
				return(-1);
			}
		}
		else {								/* end of file reached */
			fclose(fpPS);
			PSopen = FALSE;
			menu ("File", "Open.../O", itemenable);
			if (count > 0) {				/* token present */
				*sPtr = '\0';
				return(1);
			}
			else return(0);					/* no token present */
		}
		c = getc(fpPS);
	}
	*sPtr = '\0';
	return(1);								/* token present */
}


/*
 *  Copy PostScript string to s if present, return(2); else return(0) if
 *  no string not properly terminated or return(-1) if buffer overflow.
 *  Character '(' already found in input stream.
 */
 
getPSstr(s, max)
char *s;
int max;
{
	char *sPtr;
	int count = 0, excess = 0, c;

	sPtr = s;
	c = getc(fpPS);
	
	while (c != ')'  ||  excess > 0) {		/* copy character to s */
		if (c != EOF) {
			if (++count < max) {			/* copy c if space available */
				*sPtr++ = c;
				if (c == '(') ++excess;		/* increase unbalanced ( count */
				else if (c == ')') --excess;
			}
			else {							/* buffer overflow */
				*sPtr = '\0';
				return(-1);
			}
		}
		else {								/* end of file reached */
			fclose(fpPS);
			PSopen = FALSE;
			menu ("File", "Open.../O", itemenable);
			if (count > 0) {				/* string present */
				*sPtr = '\0';				/* but not terminated by ')' */
				return(-2);
			}
			else return(0);					/* no string present */
		}
		c = getc(fpPS);
	}
	*sPtr = '\0';
	return(2);								/* string present */
}


IsFloat(s)
char *s;
{
	char c;

	while (WhiteSpace(*s)) s++;
	if (*s == '-' || *s == '+') s++;	
	while (c = *s++)
		if (c < '0' || c > '9') {
			if (c == '.') break;
			else return (0);
		}
	if (c != '.') return(0);					/* no decimal point */
	while (c = *s++) if (c < '0'  ||  c > '9') return (0); /* decimal part */
	return (1);
} 


IsInteger(s)
char *s;
{
	char c;

	while (WhiteSpace(*s)) s++;
	if (*s == '-' || *s == '+') s++;	
	while (c = *s++) if (c < '0'  ||  c > '9') return (0);
	return (1);
} 

WhiteSpace(c)
{
	switch (c) {
		case ' ':
		case '\n':
		case '\t':
		case '\v':
		case '\f':
		case '\r':
			return (1);
		default:
			return (0);
	}
}